<?php
/*
 * Plugin: ultimania
 * ~~~~~~~~~~~~~~~~~
 * This plugin is made to replace the database Gerymania. Gerymania was sadly
 * the only global record database supporting the Stunts mode. This wouldn't be
 * the problem, if it would be fast and have just that basic features like
 * a ban system for cheaters (in stunt cheating is very easy :/).
 * This is the reason, why this plugin have been coded. It includes:
 * 1. A ban system
 * 2. A webversion to browse the records
 * 3. A "report" function ingame and in the browser to report cheats
 * 		(those reported records will be checked by ultimania moderators)
 * 		-- removed since 1.2.0 --
 * 4. An export function to import the records from gerymania to ultimania.
 * 		(this isn't included in the plugin, theres a script which loads the known
 * 		UIds from the database and request the top rankings)
 * 		-- gerymania is dead, but i imported everything possible --
 * 
 * ----------------------------------------------------------------------------------
 * Author:		askuri
 * Version:		See below
 * Date:		25.08.2015
 * Copyright:	2015 Martin Weber
 * Hompeage:	askuri.de
 * Xaseco:		See below
 * PHP:			See below
 * Game:		Trackmania Forever (TMF) only
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 * 
 * Manialink-IDs:
 * - ultimania_widget
 * - ultimania_window
 * 
 * 
 * Action-IDs:
 * - 5450101 = Open records window
 * - 5450102 = Close records window
 * - 5450103 = Report Map
 */

Aseco::registerEvent('onSync',						'ulti_onSync');
Aseco::registerEvent('onNewChallenge2',				'ulti_onNewChallenge2');
Aseco::registerEvent('onPlayerFinish',				'ulti_onPlayerFinish');
Aseco::registerEvent('onPlayerConnect',				'ulti_onPlayerConnect');
Aseco::registerEvent('onEndRace1',					'ulti_onEndRace1');
Aseco::registerEvent('onEverySecond',				'ulti_onEverySecond');
Aseco::registerEvent('onPlayerManialinkPageAnswer', 'ulti_onMlAnswer');
Aseco::registerEvent('onMenuLoaded', 				'ulti_onMenuLoaded');

Aseco::addChatCommand('ultirankinfo', 'Shows informations about a record', false);
Aseco::addChatCommand('ultiwindow', 'Shows informations about the current map', false);
Aseco::addChatCommand('ultiupdate', 'Updates Ultimania', false);
Aseco::addChatCommand('ultilist', 'Shows all records', false);

define ('ULTI_VERSION',		'1.3.1');
define ('ULTI_API_VERSION', '4'); // DO NOT CHANGE!!! you may get strange or outdated results so keep it as is
define ('ULTI_API_INFO', 	'http://askuri.de/ultimania/'); // This URL is permanent. It will (ok, should) never change and defines the real home of the API
define ('ULTI_ID_PREFIX', 	5450); // Change this if unexpected things happen when clicking something
define ('ULTI_MIN_PHP',		'5.4.0'); // Minimum required PHP version
define ('ULTI_MIN_XASECO',	'1.14'); // Minimum required XAseco version

global $ulti; // $ultimania_db is obsolete and will be removed in the upcoming releases! use $ulti->records instead

class Ultimania {
	private $aseco;
	public $cfg;
	public $next_refresh;
	
	public $records = array();
	
	/**************************
	 * BEGIN EVENT LISTENERS
	 **/
	public function onSync($aseco) {
		$this->aseco = $aseco;
		
		$this->doEnvironmentChecks();

		// Read Configuration
		$cfg = simplexml_load_file('ultimania.xml');
		
		$ulti_cfg['api_url']				= trim(file_get_contents(ULTI_API_INFO.'url.txt')).'TMF/'.ULTI_API_VERSION.'/index.php';
		if (!$ulti_cfg['api_url']) trigger_error('[Ultimania] Unable to get API URL from '.ULTI_API_INFO.'url.txt', E_USER_WARNING);

		// $ulti_cfg['refresh_interval']		= $cfg->refresh_interval;
		// hardcoded this. no need to change. records driven on same server get merged instantly and the same map on another server to the same time is unlikely
		$ulti_cfg['refresh_interval']		= 180; 
		// if ($ulti_cfg['refresh_interval'] < 60) trigger_error('[Ultimania] Refresh interval too low! Minimum: 60', E_USER_ERROR);
		
		$ulti_cfg['connect_timeout']		= $cfg->connect_timeout;
		
		$ulti_cfg['request_timeout']		= $cfg->request_timeout;
		
		$this->next_refresh = time(); // do it immediately, when plugin inited
		$this->cfg = $ulti_cfg;

		// Register this to the global version pool (for up-to-date checks)
		$aseco->plugin_versions[] = array(
			'plugin'	=> 'plugin.ultimania.php',
			'author'	=> 'askuri',
			'version'	=> ULTI_VERSION
		);
	}
	
	public function onNewChallenge2($aseco, $chall_item) {
		// Get newest API URL
		$this->cfg['api_url'] = trim(file_get_contents(ULTI_API_INFO.'url.txt')).'TMF/'.ULTI_API_VERSION.'/index.php';
		if (!$this->cfg['api_url']) trigger_error('[Ultimania] Unable to get API URL from '.ULTI_API_INFO.'url.txt', E_USER_WARNING);
		
		$this->records = $this->getTop();
		$this->aseco->releaseEvent('onUltimaniaRecordsLoaded', $this->records);
	}
	
	public function onPlayerFinish($aseco, $finish_item) {
		$login = $finish_item->player->login;
		$nick = $finish_item->player->nickname;
		$score = intval($finish_item->score);
		if ($finish_item->score == 0) return; // reject scores with 0 points
		
		// Check if record is better than personal best
		foreach ($this->records as $id => $entry) {
			if ($entry['login'] == $login) {
				$pb_found = true;
				
				if ($aseco->server->gameinfo->mode == 4) { // Stuntsmode
					if ($finish_item->score <= $entry['score']) {
						return; // No personal best
					} else {
						$this->records[$id]['score'] = $score; // update Score
					}
					
				} else { // Any other mode
					if ($finish_item->score >= $entry['score']) {
						return; // No personal best
					} else {
						$this->records[$id]['score'] = $score; // update Score
					}
				}
			}
		}
		
		// if there's no PB and less than 25 records (to avoid a 26th. record), add the score though
		if (!$pb_found and (count($this->records) < 25)) {
			$new_record = array(
				'login'			=> $login,
				'nick'			=> $nick,
				'score'			=> $score
			);
			$this->records[] = $new_record;
		}
		// Record better than personal best or no record until now
		
		// Send the new record
		$new_record = array(
			'login'			=> $login,
			'nick'			=> $nick,
			'score'			=> $score
		);
		$response = $this->sendRequest('playerfinish', $new_record);
		$aseco->console(7);
		// We got the response from the server. Lets look if the player is banned
		if ($response['banned'] == true) {
			// Banned, so we punish him with this spam messages every finish
			$message = ('$f00Your Records wont be saved on Ultimania because you\'re banned (Reason: '.$response['reason'].')');
			$aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
			return false;
		}
		
		$worst_rec = $this->records[count($this->records) - 1];
		// if the record is badder than the worst, it wont get into the top-list. so no sorting needed
		if ($worst_rec['score'] > $score) {
			return;
		}
		// sort records
		usort($this->records, 'ulti_sortRecordsDesc');
		
		// Not banned, so we throw event
		$aseco->releaseEvent('onUltimaniaRecord', $new_record); // Records Eyepiece actually doesnt need it, but it expects it
	}
	
	public function onPlayerConnect($aseco, $player) {
		$banned_players = $this->getBannedPlayers();
		
		// No players banned :)
		if ($banned_players == 'NULL') return;
		if (!is_array($banned_players)) return;
		
		$match = false;
		foreach ($banned_players as $id => $data) {
			if ($data['login'] == $player->login) {
				$match = $data;
				
				// display information window for the banned player
				$header = 'Ultimania global record database information:';
				$data = array();
				$data[] = array('$f00You\'re banned from the global records database Ultimania!');
				$data[] = array('');
				$data[] = array('This means:');
				$data[] = array('- You can\'t drive records anymore');
				$data[] = array('- On every join on a server, a message like "XY is banned from Ultimania" will appear');
				$data[] = array('- You always see this window on server join');
				$data[] = array('- Everyone playing with you can see, you\'re banned as long, as you\'re banned');
				$data[] = array('');
				$data[] = array('You can be unbanned by writing a formal apology for whatever you done');
				$data[] = array('(cheating, hacking, ...) to enwi2@t-online.de (the mail of the owner of Ultimania, Askuri).');
				$data[] = array('If we from Ultimania think, you regret whatever you done, we\'ll unban you.');
				$data[] = array('If not, we\'ll answer your mail but not unban you. In this case we can just say:');
				$data[] = array('Whatever you done, you hopefully don\'t do it again.');
				$data[] = array('');
				$data[] = array('Best regards,');
				$data[] = array('The whole Ultimania team');
				display_manialink($player->login, $header, array('Icons64x64_1', 'TrackInfo', -0.01), $data, array(1.1), 'OK');
			}
		}
		
		if ($match !== false) {
			$message = ('$f00The player '.$this->handleSpecialChars($player->nickname).'$z$f00 ($fff'.$player->login.'$f00) is banned from Ultimania! Reason: '.$match['reason']);
			$aseco->client->query('ChatSendServerMessage', $message);
		}
	}
	
	public function onEndRace1($aseco, $race) {
		$this->windowHide(false); // Close to all players
	}
	
	public function onEverySecond($aseco) {
		if (time() >= $this->next_refresh) {
			$this->records = $this->getTop();
			$aseco->releaseEvent('onUltimaniaRecordsLoaded', $this->records);
			
			$this->next_refresh = time() + $this->cfg['refresh_interval'];
		}
	}
	
	public function onMlAnswer($aseco, $answer) {
		switch ($answer[2]) {
			case ULTI_ID_PREFIX . 101:
				// Close Records Eyepiece windows
				$aseco->releaseEvent('onPlayerManialinkPageAnswer', array($answer[0], $answer[1], 91800));
				
				// Close standard windows like /list
				$aseco->releaseEvent('onPlayerManialinkPageAnswer', array($answer[0], $answer[1], 0));
				
				$this->windowShow($answer[1]);
				break;
			case ULTI_ID_PREFIX . 102:
				$this->windowHide($answer[1]);
				break;
			case ULTI_ID_PREFIX . 103:
				$this->reportMap($answer[1]);
				break;
			default: // special cases
			
				// Prefix for recordinfos is 2
				if (substr($answer[2], 0, strlen(ULTI_ID_PREFIX) + 1) == ULTI_ID_PREFIX . 2) {
					$rec = substr($answer[2], strlen(ULTI_ID_PREFIX) + 1);
					
					$command = array (	'params' => $rec,
										'author' => $aseco->server->players->player_list[$answer[1]] // Get the player object
									 );
					chat_ultirankinfo($aseco, $command);
					$this->windowHide($answer[1]);
				}
		}
	}
	
	/**
	 * END EVENT LISTENERS
	 **************************/
	
	
	public function doEnvironmentChecks() {
		if (version_compare(phpversion(), ULTI_MIN_PHP, '<')) {
			trigger_error('[Ultimania] Not supported PHP version ('.phpversion().')! Please update to min. version '.ULTI_MIN_PHP.'!', E_USER_ERROR);
		}
		
		if (!function_exists('curl_init')){
			trigger_error('[Ultimania] You need to enable cURL in php.ini!', E_USER_ERROR);
		}
		
		// Check for the right XAseco-Version
		if (defined('XASECO_VERSION')) {
			if ( version_compare(XASECO_VERSION, ULTI_MIN_XASECO, '<') ) {
				trigger_error('[Ultimania] Not supported XAseco version ('. XASECO_VERSION .')! Please update to min. version '.ULTI_MIN_XASECO.'!', E_USER_ERROR);
			}
		} else {
			trigger_error('[Ultimania] Can not identify the System, "XASECO_VERSION" is unset! This plugin runs only with XAseco/'. $xaseco_min_version .'+', E_USER_ERROR);
		}

		if ($this->aseco->server->getGame() != 'TMF') {
			trigger_error('[Ultimania] This plugin supports TMF only. Can not start with a "'. $this->aseco->server->getGame() .'" Dedicated-Server!', E_USER_ERROR);
		}
	}
	
	public function getTop() {
		$top = $this->sendRequest('gettop');
		
		if (empty($top)) return array(); // avoid foreach() warnings about invalid args
		
		return $top;
	}
	
	public function getAll() {
		$all = $this->sendRequest('gettop', array('limit' => 0));
		
		return $all;
	}
	
	public function getBannedPlayers() {
		return $this->sendRequest('getbannedplayers');
	}
	
	public function reportMap($login) { // Removed button for this, just keeping it for ... hmm good question :D
		$aseco = $this->aseco;
		
		$this->sendRequest('reportmap');
		
		$aseco->client->query('ChatSendServerMessageToLogin', '$0f0Thank you for supporting Ultimania!', $login);
		$aseco->client->query('ChatSendServerMessageToLogin', '$0f0An Ultimania moderator will look at the map soon', $login);
		
		$this->windowHide($login);
	}
	
	public function getNewestVersion() {
		$ver = trim(file_get_contents(ULTI_API_INFO.'tmf/version.txt'));
		
		if (!$ver) {
			trigger_error('[Ultimania] Unable to get current version information form '.ULTI_API_INFO, E_USER_WARNING);
			return false;
		} else {
			return $ver;
		}
	}
	
	public function getWelcomeWindowInfo() {
		$info = file_get_contents(ULTI_API_INFO.'tmf/description_window.txt');
		
		if (!$info) {
			trigger_error('[Ultimania] Unable to get window infobox content from '.ULTI_API_INFO, E_USER_WARNING);
			return false;
		}
		
		return $info;
	}
	
	public function sendRequest($action, $params = array()) {
		$aseco = $this->aseco;
		
		if (!$aseco->server->challenge->uid) {
			return;
		}
		
		$params_predefined = array( // These are send on all requests
			'action'		=> $action,
			'uid'			=> $aseco->server->challenge->uid,
			'mapname'		=> $aseco->server->challenge->name,
			'stunt'			=> ($aseco->server->gameinfo->mode == 4) ? 1 : 0,
			'server'		=> $aseco->server->serverlogin,
			'servername'	=> $aseco->server->nickname
		);
		
		$merged_params = array_merge($params, $params_predefined);
		
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, 				$this->cfg['api_url']);
		curl_setopt($ch, CURLOPT_USERAGENT, 		'TMF\Xaseco'.XASECO_VERSION.'\Ultimania'.ULTI_VERSION.'\API'.ULTI_API_VERSION);
		curl_setopt($ch, CURLOPT_HEADER, 			false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 	true);
		curl_setopt($ch, CURLOPT_POST, 				true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, 		$merged_params);
		curl_setopt($ch, CURLOPT_TIMEOUT, 			$ulti_cfg['connect_timeout']);
		curl_setopt($ch, CURLOPT_LOW_SPEED_TIME, 	$ulti_cfg['request_timeout']);
		
		$raw = curl_exec($ch);
		curl_close($ch);
		
		// $raw = str_replace('\\', '\\\\', $raw);
		// $raw = utf8_encode($raw);
		
		// if ($raw == 'null') return;
		
		$result = json_decode($raw, true);
		
		if (($result === null) and (!empty($raw))) {
			trigger_error('[Ultimania] Could not json_decode API response! Please report this to enwi2@t-online.de! Raw result:', E_USER_WARNING);
			$aseco->console($raw);
		}
		
		if (!empty($result['error']) ) {
			$aseco->console('[Ultimania] Error occurred on action '.$params['action'].': '. $result['error']);
		}
		
		return $result;
	}
	
	// stolen from records_eyepiece
	public function handleSpecialChars ($string) {
		// Remove links, e.g. "$(L|H|P)[...]...$(L|H|P)"
		$string = preg_replace('/\${1}(L|H|P)\[.*?\](.*?)\$(L|H|P)/i', '$2', $string);
		$string = preg_replace('/\${1}(L|H|P)\[.*?\](.*?)/i', '$2', $string);
		$string = preg_replace('/\${1}(L|H|P)(.*?)/i', '$2', $string);

		// Remove formatting $S $H $W $I $L $O $N
		$string = preg_replace('/\${1}[SHWILON]/i', '', $string);
		
		// Convert & " ' > <
		
		$string = str_replace(  array('&',	   '"', 	 "'", 	   '>',    '<'),
								array('&amp;', '&quot;', '&apos;', '&gt;', '&lt;'),
								$string
		);
		$string = stripNewlines($string);	// stripNewlines() from basic.inc.php

		return validateUTF8String($string);	// validateUTF8String() from basic.inc.php
	}
	
	// Stolen from plugin.records_eyepiece.php / undef, who stole it from basic.inc.php and adjusted it
	public function formatTime ($MwTime, $hsec = true) {
		if ($MwTime == -1) {
			return '???';
		} else {
			$hseconds = (($MwTime - (floor($MwTime/1000) * 1000)) / 10);
			$MwTime = floor($MwTime / 1000);
			$hours = floor($MwTime / 3600);
			$MwTime = $MwTime - ($hours * 3600);
			$minutes = floor($MwTime / 60);
			$MwTime = $MwTime - ($minutes * 60);
			$seconds = floor($MwTime);
			if ($hsec) {
				if ($hours) {
					return sprintf('%d:%02d:%02d.%02d', $hours, $minutes, $seconds, $hseconds);
				} else {
					return sprintf('%d:%02d.%02d', $minutes, $seconds, $hseconds);
				}
			} else {
				if ($hours) {
					return sprintf('%d:%02d:%02d', $hours, $minutes, $seconds);
				} else {
					return sprintf('%d:%02d', $minutes, $seconds);
				}
			}
		}
	}
	
	public function handleRecordTime($timestamp) {
		// Geryimports were done on 2013/24/03 and other dates. I keep searching them
		if ($timestamp > 1364148372 and  $timestamp <= 1367798399) {
			$time = 'Gerymania Import';
		} else {
			$time = date('M j Y G:i:s', $timestamp).'(CET)';
		}
		
		return $time;
	}
	
	public function windowShow($login) {
		$aseco = $this->aseco;
		$ultimania_db = $this->records;
		$ulti_cfg = $this->cfg;
		
		$xml = '<manialink id="ulitmania_window">
			
			<quad posn="0 3 0" sizen="79 54" bgcolor="001a" halign="center" valign="center" />
			<quad posn="0 3 2" sizen="81 56" style="Bgs1InRace" substyle="BgCard3" halign="center" valign="center" />
			
			<quad posn="0 29.5 1" sizen="82 5" style="BgsPlayerCard" substyle="BgRacePlayerLine" halign="center" />
			<quad posn="0 30 0.5" sizen="80 5" style="Bgs1InRace" substyle="BgTitle3_3" halign="center" />
			<label posn="0 29 2" style="TextRankingsBig" halign="center" text="Ultimania World Records" />
			
			<quad posn="33.5 30 3" sizen="5 5" style="Icons128x128_1" substyle="BackFocusable" action="5450102" />
			
			<label posn="-38 -23.65 3" text="$000$tUltimania TMF v'.ULTI_VERSION.' using API '.ULTI_API_VERSION.'" textsize="0" />
		';
		
		$ultinfo = $this->getWelcomeWindowInfo();
		
		$xml.= '
			<quad posn="-38 1 1" sizen="35.5 47" style="BgsPlayerCard" substyle="BgRacePlayerName" valign="center" />
			<quad posn="-2 24.5 1" sizen="40 28.25" style="BgsPlayerCard" substyle="BgRacePlayerName" />
			
			<!-- DO	NOT REMOVE THESE BANNERS!!! THE HOSTING OF ULTIMANIA IS PAYED WITH HELP OF THESE PARTNER-LINKS! -->
			<!-- So please, please keep them as they are, thanks! -->
			<quad posn="-2 -4.5 3" sizen="44 20" image="http://www.netcup.de/bilder/banner/netcup-hlogo-b440h200.png" scale="0.9" />
			<quad posn="-2 -4.5 2" sizen="40 18" style="BgsPlayerCard" substyle="BgRacePlayerName" />
			<quad posn="-2 -4.5 1" sizen="40 18" style="BgsPlayerCard" substyle="ProgressBar" url="http://askuri.de/nc_gutscheine.php" />
			<!-- BANNERS ENDING HERE -->
			
			<label posn="-1.5 24 2" text="'.$ultinfo.'" textsize="2" />
			
			<!--
			<label posn="5 -16.5 2" text="Cheated record? Report this Map!" style="CardButtonMedium" action="5450103" />
			<quad posn="-2 -14.5 1" sizen="40 8" style="BgsPlayerCard" substyle="BgRacePlayerName" />
			-->
		';
		
		$xml.= '<frame posn="-38.5 26.5 1">';
		
		if (!isset($ultimania_db[0])) {
			$xml.= '<label posn="2 -5 1" text="$oCurrently'.CRLF.'no records :(" textsize="2" />';
		} else {
			$x = 0;
			$y = -3;
			$recs_count = count($ultimania_db);
			foreach ($ultimania_db as $rank => $data) {
				$rank++;
				
				$type = $aseco->server->gameinfo->mode;
				
				if ($type == 4) {
					$score = $data['score'];
				} else {
					$score = $this->formatTime($data['score']);
				}
				
				switch ($rank) {
					case 1:
						$xml.= '<quad posn="'.($x + 1.5).' '.$y.' 1" sizen="1.5 1.5" style="Icons64x64_1" substyle="First" />';
						break;
					case 2:
						$xml.= '<quad posn="'.($x + 1.5).' '.$y.' 1" sizen="1.5 1.5" style="Icons64x64_1" substyle="Second" />';
						break;
					case 3:
						$xml.= '<quad posn="'.($x + 1.5).' '.$y.' 1" sizen="1.5 1.5" style="Icons64x64_1" substyle="Third" />';
						break;
					
					default:
						$xml.= '<label posn="'.($x + 1.2).' '.$y.' 1" text="'.$rank.'." textsize="1" />';
				}
				
				$xml.= '<label posn="'.($x + 4).' '.$y.' 1" text="'.$score.'" textsize="1" />';
				$xml.= '<label posn="'.($x + 8).' '.$y.' 1" text="'.$this->handleSpecialChars($data['nick']).'" sizen="12 2" textsize="1" />';
				$xml.= '<label posn="'.($x + 21).' '.$y.' 1" text="'.$data['login'].'" sizen="13 2" textsize="1" />';
				
				if ($data['login'] == $login) {
					$xml.= '<quad posn="'.($x + 0.7).' '.($y + 0.2).' 0.5" sizen="35 2" style="BgsPlayerCard" substyle="ProgressBar" />';
				}
				
				$xml.= '<quad posn="'.($x + 34.4).' '.($y - 0.1).' 2" sizen="1.35 1.35" style="Icons64x64_1" substyle="TrackInfo" action="'. ULTI_ID_PREFIX . 2 . $rank.'" />';
				// $xml.= '<quad posn="'.($x + 16).' '.($y).' 1" sizen="2 2" style="BgsPlayerCard" substyle="ProgressBar" />';
				
				$y -= 1.83; // 1,75
				
				if ($rank == 25) {
					$x+= 18;
					$y = -3;
				}
			}
		}
		
		$xml.= '</frame> </manialink>';
			
		$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($login, $xml, 0, false));
	}

	public function windowHide($login) {
		$aseco = $this->aseco;
		
		$xml = '<manialink version="1" id="ulitmania_window"></manialink>';
		
		if ($login === false) {
			$aseco->client->addCall('SendDisplayManialinkPage', array($xml, 0, false));	
		} else {
			$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($login, $xml, 0, false));
		}
	}
}

/**
 * Chatcommands
 **/

function chat_ultirankinfo($aseco, $command) {
	global $ulti;
	
	$ultimania_db = $ulti->records;
	$player = $command['author'];
	
	if (!is_numeric($command['params'])) {
		$aseco->client->query('ChatSendServerMessageToLogin', '$ff0>> $f00Usage: $bbb/ultirankinfo ranknumber$f00  (e.g. $bbb/ultirankinfo 1$f00)', $player->login);
		return;
	}
	
	$rank = $command['params'] -= 1;
	
	if (!isset($ultimania_db[$rank])) {
		$aseco->client->query('ChatSendServerMessageToLogin', '$ff0>> $f00This record does not exists', $player->login);
		return;
	}
	
	$time = $ulti->handleRecordTime($ultimania_db[$rank]['add_time']);
	
	$header = 'Ultimania record information:';
	
	$player->msgs = array();
	$player->msgs[0] = array(1, // startpage
							$header,
							array(0.8, 0.3, 0.5), // widths: overall, col1, col2, col3, ...
							array('Icons64x64_1', 'TrackInfo') // icon style
					   );

	$player->msgs[] = array(array('Score', 			'$FFC'.$ultimania_db[$rank]['score'].' Points'),
							array('Login', 			'$FFC'.$ultimania_db[$rank]['login']),
							array('Nickname',		'$FFC'.$ultimania_db[$rank]['nick']),
							array('Recorded on', 	'$FFC'.$time),
							array('$h[ulti:admin_rec?uid='.$aseco->server->challenge->uid.'&login='.$ultimania_db[$command['params']]['login'].']Admin')
							);
	/* no second page
	$player->msgs[] = array(array('page 2', 'row 1', 'col 3'),
							array('page 2', 'row 2', 'col 3')
							);
	*/

	// display ManiaLink message
	display_manialink_multi($player);
}

function chat_ultiwindow($aseco, $command) {
	global $ulti;
	
	$login = $command['author']->login;
	$ulti->windowShow($login);
}

function chat_ultiupdate($aseco, $command) {
	global $ulti;
	
	$login = $command['author']->login;
	if ($aseco->isMasterAdminL($login)) {
		$newest = $ulti->getNewestVersion();
		if (version_compare(ULTI_VERSION, $newest) == -1) {
			$content = file_get_contents(ULTI_API_INFO.'tmf/versions/'.$newest.'.php_');
			
			if ($content) {
				if (!file_put_contents(getcwd().'/plugins/plugin.ultimania.php', $content)) {
					$aseco->client->query('ChatSendServerMessageToLogin', '$f00Unable to replace "plugin.ultimania.php". Please check the file-rights and try again.', $login);
				} else {
					$aseco->client->query('ChatSendServerMessageToLogin', '$0f0Successfully updated. Please restart XAseco', $login);
				}
			} else {
				$aseco->client->query('ChatSendServerMessageToLogin', '$f00Getting newest version file failed!', $login);
			}
		} else {
			$aseco->client->query('ChatSendServerMessageToLogin', '$0f0No update available', $login);
		}
	} else {
		$aseco->client->query('ChatSendServerMessageToLogin', '$f00No permissions!', $login);
	}
}

function chat_ultilist($aseco, $command) {
	global $ulti;
	
	$player = $command['author'];
	$recs = $ulti->getAll();
	
	if (!isset($recs[0])) {
		$aseco->client->query('ChatSendServerMessageToLogin', '$06fNo Records. Feel free to drive the first :)', $player->login);
		return;
	}
	
	$header = 'Ultimania records:'; // Window Header
	
	$player->msgs = array();
	
	// Settings
	$player->msgs[0] = array(1, // startpage
							$header,
							array(1.3, 0.1, 0.15, 0.3, 0.35, 0.4), // widths: overall, col1, col2, col3, ...
							array('Icons64x64_1', 'TrackInfo') // icon style
					   );
	$page = 1;
	foreach ($recs as $id => $rec) {
		$time = $ulti->handleRecordTime($rec['add_time']);
		$row = array($id + 1, $rec['score'], $rec['nick'], $rec['login'], $time);
		
		$player->msgs[$page][] = $row;
		
		if ((($id + 1) % 15) == 0) $page++;
	}

	// display ManiaLink message
	display_manialink_multi($player);
}

/**
 * Sorting records descending
 * e.g. usort($array, 'ulti_sortRecordsDesc')
 **/
function ulti_sortRecordsDesc($a, $b) {
	global $aseco;
	
	if ($a['score'] == $b['score']) {
		return 0;
	}
	
	if ($aseco->server->gameinfo->mode == 4) { // Stunt
		return ($a['score'] > $b['score']) ? -1 : 1;
	} else { // other mode
		return ($a['score'] < $b['score']) ? -1 : 1;
	} 
}

/**
 * Passing Callbacks to the Ultimania class
 **/
function ulti_onSync($aseco) {
	global $ulti;
	$ulti->onSync($aseco);
}
function ulti_onNewChallenge2($aseco, $challenge_item) {
	global $ulti;
	$ulti->onNewChallenge2($aseco, $challenge_item);
}
function ulti_onPlayerFinish($aseco, $finish) {
	global $ulti;
	$ulti->onPlayerFinish($aseco, $finish);
}
function ulti_onPlayerConnect($aseco, $player) {
	global $ulti;
	$ulti->onPlayerConnect($aseco, $player);
}
function ulti_onEndRace1($aseco, $race) {
	global $ulti;
	$ulti->onEndRace1($aseco, $race);
}
function ulti_onEverySecond($aseco) {
	global $ulti;
	$ulti->onEverySecond($aseco);
}
function ulti_onMlAnswer($aseco, $answer) {
	global $ulti;
	$ulti->onMlAnswer($aseco, $answer);
}
function ulti_onMenuLoaded($aseco, $menu) {
	global $ulti;
	
	// Doc: http://fish.stabb.de/index.php5?page=downloads&subpage=148
	$menu->addEntry('tracks', 'fulllist', true, 'Ultimania', 'ultifolder', '');
	
	$menu->addEntry('', 'tracks', false, 'Ultimania', 'ulti');
	$menu->addEntry('ulti', '', true, 'Full recordlist', 'ulti_fulllist', '/ultilist');
	$menu->addEntry('ulti', '', true, 'Open window', 'ulti_openwindow', '/ultiwindow');
	$menu->addEntry('ulti', '', true, '$ff0Update plugin', 'ulti_openwindow', '/ultiupdate', 'MasterAdmin');
}

$ulti = new Ultimania;
?>
